@@ -221,19 +221,14 @@ public:
      */
     void UpdatePins();
 
-    /**
-     * Return true if the given unit \a aUnit has a display name set.
-     *
-     * @return true if the display name of a unit is set, otherwise false.
-     */
-    bool HasUnitDisplayName( int aUnit ) const;
-
     /**
      * Return the display name for a given unit \a aUnit.
      *
      * @return the display name of a unit if set, or the ordinal name of the unit otherwise.
      */
-    wxString GetUnitDisplayName( int aUnit ) const;
+    wxString GetUnitDisplayName( int aUnit, bool aLabel ) const override;
+
+    wxString GetBodyStyleDescription( int aBodyStyle, bool aLabel ) const override;
 
     void SetBodyStyle( int aBodyStyle ) override;
 
@@ -510,6 +505,7 @@ public:
     }
 
     void SetRefProp( const wxString& aRef );
+
     wxString GetValueProp() const
     {
         return GetValue( false, &Schematic()->CurrentSheet(), false );
@@ -520,47 +516,32 @@ public:
         SetValueFieldText( aRef );
     }
 
-    wxString GetUnitProp() const
+    int GetUnitProp() const
     {
-        int unit = GetUnitSelection( &Schematic()->CurrentSheet() );
-
-        if( HasUnitDisplayName( unit ) )
-            return GetUnitDisplayName( unit );
-        else
-            return SubReference( unit, false );
+        return GetUnitSelection( &Schematic()->CurrentSheet() );
     }
 
-    void SetUnitProp( const wxString& aUnit )
+    void SetUnitProp( int aUnit )
     {
-        for( int unit = 1; unit <= GetUnitCount(); unit++ )
+        SetUnitSelection( &Schematic()->CurrentSheet(), aUnit );
+        SetUnit( aUnit );
+    }
+
+    wxString GetBodyStyleProp() const override
+    {
+        return GetBodyStyleDescription( GetBodyStyle(), false );
+    }
+
+    void SetBodyStyleProp( const wxString& aBodyStyle ) override
+    {
+        for( int bodyStyle : { BODY_STYLE::BASE, BODY_STYLE::DEMORGAN } )
         {
-            if( HasUnitDisplayName( unit ) && GetUnitDisplayName( unit ) == aUnit )
+            if( GetBodyStyleDescription( bodyStyle, false ) == aBodyStyle )
             {
-                SetUnitSelection( &Schematic()->CurrentSheet(), unit );
-                SetUnit( unit );
+                SetBodyStyle( bodyStyle );
                 return;
             }
         }
-
-        for( int unit = 1; unit <= GetUnitCount(); unit++ )
-        {
-            if( SubReference( unit, false ) == aUnit )
-            {
-                SetUnitSelection( &Schematic()->CurrentSheet(), unit );
-                SetUnit( unit );
-                return;
-            }
-        }
-    }
-
-    int GetBodyStyleProp() const
-    {
-        return GetBodyStyle();
-    }
-
-    void SetBodyStyleProp( int aBodyStyle )
-    {
-        SetBodyStyle( aBodyStyle );
     }
 
     /**
@@ -709,6 +690,12 @@ public:
     /// Set the selected unit of this symbol for all sheets.
     void SetUnitSelection( int aUnitSelection );
 
+    /**
+     * SCH_SYMBOLs don't currently support embedded files, but their LIB_SYMBOL counterparts
+     * do.
+     */
+    EMBEDDED_FILES* GetEmbeddedFiles() override;
+
     void Move( const VECTOR2I& aMoveVector ) override
     {
         if( aMoveVector == VECTOR2I( 0, 0 ) )
