@@ -409,6 +409,8 @@ struct ACOMPONENT6
     bool         locked;
     bool         nameon;
     bool         commenton;
+    wxString     sourceUniqueID;
+    wxString     sourceHierachicalPath;
     wxString     sourcedesignator;
     wxString     sourcefootprintlibrary;
     wxString     pattern;
@@ -598,14 +600,14 @@ struct ACOMPONENTBODY6
     int                  kind             = 0;
     int                  subpolyindex     = 0;
     int                  unionindex       = 0;
-    int                  arc_resolution   = 0;;
+    int                  arc_resolution   = 0;
     bool                 is_shape_based   = false;
     int                  cavity_height    = 0;
     int                  standoff_height  = 0;
     int                  overall_height   = 0;
     int                  body_projection  = 0;
     int                  body_color_3d    = 0;
-    int                  body_opacity_3d  = 0;
+    double               body_opacity_3d  = 0.0;
     wxString             identifier;
     wxString             texture;
     int                  texture_center_x = 0;
@@ -679,6 +681,9 @@ struct APAD6
     ALTIUM_LAYER tolayer;
     ALTIUM_LAYER fromlayer;
 
+    int32_t pad_to_die_length;
+    int32_t pad_to_die_delay;
+
     std::unique_ptr<APAD6_SIZE_AND_SHAPE> sizeAndShape;
 
     explicit APAD6( ALTIUM_BINARY_PARSER& aReader );
