@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2013-2017 CERN
- * Copyright (C) 2017-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
@@ -30,11 +30,13 @@
 #include <board.h>
 #include <board_commit.h>
 #include <gal/graphics_abstraction_layer.h>
+#include <geometry/geometry_utils.h>
 #include <pad.h>
 #include <pcb_group.h>
 #include <pcb_generator.h>
 #include <pcb_edit_frame.h>
 #include <spread_footprints.h>
+#include <tool/tool_manager.h>
 #include <tools/pcb_actions.h>
 #include <tools/pcb_selection_tool.h>
 #include <tools/edit_tool.h>
@@ -48,6 +50,7 @@
 #include <drc/drc_item.h>
 #include <drc/drc_rule.h>
 #include <drc/drc_interactive_courtyard_clearance.h>
+#include <view/view_controls.h>
 
 
 int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
@@ -96,10 +99,14 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
 
     for( size_t i = 0; i < sorted.size() - 1; i++ )
     {
-        BOARD_ITEM* a = dynamic_cast<BOARD_ITEM*>( sorted[i] );
-        BOARD_ITEM* b = dynamic_cast<BOARD_ITEM*>( sorted[( i + 1 ) % sorted.size()] );
+        EDA_ITEM* edaItemA = sorted[i];
+        EDA_ITEM* edaItemB = sorted[( i + 1 ) % sorted.size()];
 
-        wxCHECK2( a && b, continue );
+        if( !edaItemA->IsBOARD_ITEM() || !edaItemB->IsBOARD_ITEM() )
+            continue;
+
+        BOARD_ITEM* a = static_cast<BOARD_ITEM*>( edaItemA );
+        BOARD_ITEM* b = static_cast<BOARD_ITEM*>( edaItemB );
 
         // Swap X,Y position
         VECTOR2I aPos = a->GetPosition(), bPos = b->GetPosition();
@@ -121,8 +128,8 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
             // Flip both if needed
             if( aFP->IsFlipped() != bFP->IsFlipped() )
             {
-                aFP->Flip( aPos, false );
-                bFP->Flip( bPos, false );
+                aFP->Flip( aPos, FLIP_DIRECTION::TOP_BOTTOM );
+                bFP->Flip( bPos, FLIP_DIRECTION::TOP_BOTTOM );
             }
 
             // Set orientation
@@ -192,13 +199,13 @@ int EDIT_TOOL::PackAndMoveFootprints( const TOOL_EVENT& aEvent )
     {
         commit.Modify( item );
         item->SetFlags( IS_MOVING );
-        footprintsBbox.Merge( item->GetBoundingBox( false, false ) );
+        footprintsBbox.Merge( item->GetBoundingBox( false ) );
     }
 
     SpreadFootprints( &footprintsToPack, footprintsBbox.Normalize().GetOrigin(), false );
 
     if( doMoveSelection( aEvent, &commit, true ) )
-        commit.Push( _( "Pack footprints" ) );
+        commit.Push( _( "Pack Footprints" ) );
     else
         commit.Revert();
 
@@ -216,13 +223,19 @@ int EDIT_TOOL::Move( const TOOL_EVENT& aEvent )
 
     if( BOARD_COMMIT* commit = dynamic_cast<BOARD_COMMIT*>( aEvent.Commit() ) )
     {
-        wxCHECK( aEvent.SynchronousState(), 0 );
-        aEvent.SynchronousState()->store( STS_RUNNING );
+        // Most moves will be synchronous unless they are coming from the API
+        if( aEvent.SynchronousState() )
+            aEvent.SynchronousState()->store( STS_RUNNING );
 
         if( doMoveSelection( aEvent, commit, true ) )
-            aEvent.SynchronousState()->store( STS_FINISHED );
-        else
+        {
+            if( aEvent.SynchronousState() )
+                aEvent.SynchronousState()->store( STS_FINISHED );
+        }
+        else if( aEvent.SynchronousState() )
+        {
             aEvent.SynchronousState()->store( STS_CANCELLED );
+        }
     }
     else
     {
@@ -248,48 +261,51 @@ VECTOR2I EDIT_TOOL::getSafeMovement( const VECTOR2I& aMovement, const BOX2I& aSo
 {
     typedef std::numeric_limits<int> coord_limits;
 
-    int max = coord_limits::max();
-    int min = -max;
+    static const double max = coord_limits::max() - (int) COORDS_PADDING;
+    static const double min = -max;
 
-    double left = aBBoxOffset.x + aSourceBBox.GetPosition().x;
-    double top = aBBoxOffset.y + aSourceBBox.GetPosition().y;
-
-    double right = left + aSourceBBox.GetSize().x;
-    double bottom = top + aSourceBBox.GetSize().y;
+    BOX2D testBox( aSourceBBox.GetPosition(), aSourceBBox.GetSize() );
+    testBox.Offset( aBBoxOffset );
 
     // Do not restrict movement if bounding box is already out of bounds
-    if( left < min || top < min || right > max || bottom > max )
+    if( testBox.GetLeft() < min || testBox.GetTop() < min || testBox.GetRight() > max
+        || testBox.GetBottom() > max )
+    {
         return aMovement;
+    }
 
-    // Constrain moving bounding box to coordinates limits
-    VECTOR2D tryMovement( aMovement );
-    VECTOR2D bBoxOrigin( aSourceBBox.GetPosition() + aBBoxOffset );
-    VECTOR2D clampedBBoxOrigin = GetClampedCoords( bBoxOrigin + tryMovement, COORDS_PADDING );
+    testBox.Offset( aMovement );
 
-    tryMovement = clampedBBoxOrigin - bBoxOrigin;
+    if( testBox.GetLeft() < min )
+        testBox.Offset( min - testBox.GetLeft(), 0 );
 
-    VECTOR2D bBoxEnd( aSourceBBox.GetEnd() + aBBoxOffset );
-    VECTOR2D clampedBBoxEnd = GetClampedCoords( bBoxEnd + tryMovement, COORDS_PADDING );
+    if( max < testBox.GetRight() )
+        testBox.Offset( -( testBox.GetRight() - max ), 0 );
 
-    tryMovement = clampedBBoxEnd - bBoxEnd;
+    if( testBox.GetTop() < min )
+        testBox.Offset( 0, min - testBox.GetTop() );
 
-    return GetClampedCoords<double, int>( tryMovement );
+    if( max < testBox.GetBottom() )
+        testBox.Offset( 0, -( testBox.GetBottom() - max ) );
+
+    return KiROUND( testBox.GetPosition() - aBBoxOffset - aSourceBBox.GetPosition() );
 }
 
 
 bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit, bool aAutoStart )
 {
-    bool moveWithReference = aEvent.IsAction( &PCB_ACTIONS::moveWithReference );
-    bool moveIndividually = aEvent.IsAction( &PCB_ACTIONS::moveIndividually );
+    const bool moveWithReference = aEvent.IsAction( &PCB_ACTIONS::moveWithReference );
+    const bool moveIndividually = aEvent.IsAction( &PCB_ACTIONS::moveIndividually );
 
-    PCB_BASE_EDIT_FRAME*  editFrame = getEditFrame<PCB_BASE_EDIT_FRAME>();
-    PCBNEW_SETTINGS*      cfg = editFrame->GetPcbNewSettings();
-    BOARD*                board = editFrame->GetBoard();
-    KIGFX::VIEW_CONTROLS* controls  = getViewControls();
-    VECTOR2I              originalCursorPos = controls->GetCursorPosition();
-    STATUS_TEXT_POPUP     statusPopup( frame() );
-    wxString              status;
-    size_t                itemIdx = 0;
+    PCB_BASE_EDIT_FRAME*               editFrame = getEditFrame<PCB_BASE_EDIT_FRAME>();
+    PCBNEW_SETTINGS*                   cfg = editFrame->GetPcbNewSettings();
+    BOARD*                             board = editFrame->GetBoard();
+    KIGFX::VIEW_CONTROLS*              controls = getViewControls();
+    VECTOR2I                           originalCursorPos = controls->GetCursorPosition();
+    VECTOR2I                           originalMousePos = controls->GetMousePosition();
+    std::unique_ptr<STATUS_TEXT_POPUP> statusPopup;
+    wxString                           status;
+    size_t                             itemIdx = 0;
 
     // Be sure that there is at least one item that we can modify. If nothing was selected before,
     // try looking for the stuff under mouse cursor (i.e. KiCad old-style hover selection)
@@ -299,12 +315,14 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                 sTool->FilterCollectorForMarkers( aCollector );
                 sTool->FilterCollectorForHierarchy( aCollector, true );
                 sTool->FilterCollectorForFreePads( aCollector );
+                sTool->FilterCollectorForTableCells( aCollector );
             },
             true /* prompt user regarding locked items */ );
 
     if( m_dragging || selection.Empty() )
         return false;
 
+    TOOL_EVENT pushedEvent = aEvent;
     editFrame->PushTool( aEvent );
     Activate();
 
@@ -314,10 +332,16 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
     controls->ForceCursorPosition( false );
 
     auto displayConstraintsMessage =
-            [editFrame]( bool constrained )
+            [editFrame]( bool aHV45Enabled )
             {
-                editFrame->DisplayConstraintsMsg( constrained ? _( "Constrain to H, V, 45" )
-                                                              : wxString( wxT( "" ) ) );
+                wxString msg;
+
+                if( aHV45Enabled )
+                    msg = _( "Angle snap guides: H, V, 45°" );
+                else
+                    msg.clear();
+
+                editFrame->DisplayConstraintsMsg( msg );
             };
 
     auto updateStatusPopup =
@@ -343,7 +367,10 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                     msg = item->GetTypeDesc().Lower();
                 }
 
-                statusPopup.SetText( wxString::Format( popuptext, msg, ii, count ) );
+                if( !statusPopup )
+                    statusPopup = std::make_unique<STATUS_TEXT_POPUP>( frame() );
+
+                statusPopup->SetText( wxString::Format( popuptext, msg, ii, count ) );
             };
 
     std::vector<BOARD_ITEM*> sel_items;         // All the items operated on by the move below
@@ -351,16 +378,20 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
 
     for( EDA_ITEM* item : selection )
     {
-        if( BOARD_ITEM* boardItem = dynamic_cast<BOARD_ITEM*>( item ) )
+        if( item->IsBOARD_ITEM() )
         {
+            BOARD_ITEM* boardItem = static_cast<BOARD_ITEM*>( item );
+
             if( !selection.IsHover() )
                 orig_items.push_back( boardItem );
 
             sel_items.push_back( boardItem );
         }
 
-        if( FOOTPRINT* footprint = dynamic_cast<FOOTPRINT*>( item ) )
+        if( item->Type() == PCB_FOOTPRINT_T )
         {
+            FOOTPRINT* footprint = static_cast<FOOTPRINT*>( item );
+
             for( PAD* pad : footprint->Pads() )
                 sel_items.push_back( pad );
 
@@ -378,7 +409,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
         if( selection.IsHover() )
             m_toolMgr->RunAction( PCB_ACTIONS::selectionClear );
 
-        editFrame->PopTool( aEvent );
+        editFrame->PopTool( pushedEvent );
         return false;
     }
 
@@ -388,14 +419,14 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
 
         for( EDA_ITEM* item : selection.GetItemsSortedBySelectionOrder() )
         {
-            if( BOARD_ITEM* boardItem = dynamic_cast<BOARD_ITEM*>( item ) )
-                orig_items.push_back( boardItem );
+            if( item->IsBOARD_ITEM() )
+                orig_items.push_back( static_cast<BOARD_ITEM*>( item ) );
         }
 
         updateStatusPopup( orig_items[ itemIdx ], itemIdx + 1, orig_items.size() );
-        statusPopup.Popup();
-        statusPopup.Move( KIPLATFORM::UI::GetMousePosition() + wxPoint( 20, 20 ) );
-        canvas()->SetStatusPopup( statusPopup.GetPanel() );
+        statusPopup->Popup();
+        statusPopup->Move( KIPLATFORM::UI::GetMousePosition() + wxPoint( 20, 20 ) );
+        canvas()->SetStatusPopup( statusPopup->GetPanel() );
 
         m_selectionTool->ClearSelection();
         m_selectionTool->AddItemToSel( orig_items[ itemIdx ] );
@@ -405,23 +436,27 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
     }
 
     bool            restore_state = false;
-    VECTOR2I        originalPos;
-    VECTOR2I        totalMovement;
+    VECTOR2I        originalPos = originalCursorPos;  // Initialize to current cursor position
     VECTOR2D        bboxMovement;
     BOX2I           originalBBox;
     bool            updateBBox = true;
-    LSET            layers( editFrame->GetActiveLayer() );
+    LSET            layers( { editFrame->GetActiveLayer() } );
     PCB_GRID_HELPER grid( m_toolMgr, editFrame->GetMagneticItemsSettings() );
     TOOL_EVENT      copy = aEvent;
     TOOL_EVENT*     evt = &copy;
     VECTOR2I        prevPos;
     bool            enableLocalRatsnest = true;
 
-    bool hv45Mode        = false;
+    bool hv45Enabled = Is45Limited();
     bool eatFirstMouseUp = true;
     bool allowRedraw3D   = cfg->m_Display.m_Live3DRefresh;
     bool showCourtyardConflicts = !m_isFootprintEditor && cfg->m_ShowCourtyardCollisions;
 
+    // Axis locking for arrow key movement
+    enum class AXIS_LOCK { NONE, HORIZONTAL, VERTICAL };
+    AXIS_LOCK axisLock = AXIS_LOCK::NONE;
+    long      lastArrowKeyAction = 0;
+
     // Used to test courtyard overlaps
     std::unique_ptr<DRC_INTERACTIVE_COURTYARD_CLEARANCE> drc_on_move = nullptr;
 
@@ -432,7 +467,29 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
         drc_on_move->Init( board );
     }
 
-    displayConstraintsMessage( hv45Mode );
+    auto configureAngleSnap =
+            [&]( bool aHV45Enabled )
+            {
+                std::vector<VECTOR2I> directions;
+
+                if( aHV45Enabled )
+                {
+                    // H, V, and both 45° diagonals
+                    directions = { VECTOR2I( 1, 0 ), VECTOR2I( 0, 1 ),
+                                   VECTOR2I( 1, 1 ), VECTOR2I( 1, -1 ) };
+                }
+                // Not enabled = no directions = no guides
+
+                grid.SetSnapLineDirections( directions );
+
+                if( !directions.empty() )
+                    grid.SetSnapLineOrigin( originalPos );
+                else
+                    grid.ClearSnapLine();
+            };
+
+    configureAngleSnap( hv45Enabled );
+    displayConstraintsMessage( hv45Enabled );
 
     // Prime the pump
     m_toolMgr->PostAction( ACTIONS::refreshPreview );
@@ -441,6 +498,14 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
     do
     {
         VECTOR2I movement;
+
+        if( ( hv45Enabled != Is45Limited() ) )
+        {
+            hv45Enabled = Is45Limited();
+            configureAngleSnap( hv45Enabled );
+            displayConstraintsMessage( hv45Enabled );
+        }
+
         editFrame->GetCanvas()->SetCurrentCursor( KICURSOR::MOVING );
         grid.SetSnap( !evt->Modifier( MD_SHIFT ) );
         grid.SetUseGrid( getView()->GetGAL()->GetGridSnapping() && !evt->DisableGridSnapping() );
@@ -450,41 +515,85 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
         if( evt->IsMotion() || evt->IsDrag( BUT_LEFT ) )
             eatFirstMouseUp = false;
 
-        if( evt->IsAction( &PCB_ACTIONS::move ) || evt->IsMotion() || evt->IsDrag( BUT_LEFT )
+        if( evt->IsAction( &PCB_ACTIONS::move )
+                || evt->IsMotion()
+                || evt->IsDrag( BUT_LEFT )
                 || evt->IsAction( &ACTIONS::refreshPreview )
                 || evt->IsAction( &PCB_ACTIONS::moveWithReference )
                 || evt->IsAction( &PCB_ACTIONS::moveIndividually ) )
         {
-            if( m_dragging && evt->Category() == TC_MOUSE )
+            if( m_dragging && (    evt->IsMotion()
+                                || evt->IsDrag( BUT_LEFT )
+                                || evt->IsAction( &ACTIONS::refreshPreview ) ) )
             {
                 bool redraw3D = false;
 
-                VECTOR2I mousePos( controls->GetMousePosition() );
+                GRID_HELPER_GRIDS selectionGrid = grid.GetSelectionGrid( selection );
 
-                m_cursor = grid.BestSnapAnchor( mousePos, layers,
-                                                grid.GetSelectionGrid( selection ), sel_items );
-
-                if( controls->GetSettings().m_lastKeyboardCursorPositionValid )
+                // We need to bypass refreshPreview action here because it is triggered by the move,
+                // so we were getting double-key events that toggled the axis locking if you
+                // pressed them in a certain order.
+                if( controls->GetSettings().m_lastKeyboardCursorPositionValid && ! evt->IsAction( &ACTIONS::refreshPreview ) )
                 {
+                    VECTOR2I keyboardPos( controls->GetSettings().m_lastKeyboardCursorPosition );
                     long action = controls->GetSettings().m_lastKeyboardCursorCommand;
 
-                    // The arrow keys are by definition SINGLE AXIS.  Do not allow the other
-                    // axis to be snapped to the grid.
+                    grid.SetSnap( false );
+                    m_cursor = grid.Align( keyboardPos, selectionGrid );
+
+                    // Update axis lock based on arrow key press
                     if( action == ACTIONS::CURSOR_LEFT || action == ACTIONS::CURSOR_RIGHT )
-                        m_cursor.y = prevPos.y;
+                    {
+                        if( axisLock == AXIS_LOCK::HORIZONTAL )
+                        {
+                            // Check if opposite horizontal key pressed to unlock
+                            if( ( lastArrowKeyAction == ACTIONS::CURSOR_LEFT && action == ACTIONS::CURSOR_RIGHT ) ||
+                                ( lastArrowKeyAction == ACTIONS::CURSOR_RIGHT && action == ACTIONS::CURSOR_LEFT ) )
+                            {
+                                axisLock = AXIS_LOCK::NONE;
+                            }
+                            // Same direction axis, keep locked
+                        }
+                        else
+                        {
+                            axisLock = AXIS_LOCK::HORIZONTAL;
+                        }
+                    }
                     else if( action == ACTIONS::CURSOR_UP || action == ACTIONS::CURSOR_DOWN )
-                        m_cursor.x = prevPos.x;
+                    {
+                        if( axisLock == AXIS_LOCK::VERTICAL )
+                        {
+                            // Check if opposite vertical key pressed to unlock
+                            if( ( lastArrowKeyAction == ACTIONS::CURSOR_UP && action == ACTIONS::CURSOR_DOWN ) ||
+                                ( lastArrowKeyAction == ACTIONS::CURSOR_DOWN && action == ACTIONS::CURSOR_UP ) )
+                            {
+                                axisLock = AXIS_LOCK::NONE;
+                            }
+                            // Same direction axis, keep locked
+                        }
+                        else
+                        {
+                            axisLock = AXIS_LOCK::VERTICAL;
+                        }
+                    }
+
+                    lastArrowKeyAction = action;
                 }
+                else
+                {
+                    VECTOR2I mousePos( controls->GetMousePosition() );
+
+                    m_cursor = grid.BestSnapAnchor( mousePos, layers, selectionGrid, sel_items );
+                }
+
+                if( axisLock == AXIS_LOCK::HORIZONTAL )
+                    m_cursor.y = prevPos.y;
+                else if( axisLock == AXIS_LOCK::VERTICAL )
+                    m_cursor.x = prevPos.x;
 
                 if( !selection.HasReferencePoint() )
                     originalPos = m_cursor;
 
-                if( hv45Mode )
-                {
-                    VECTOR2I moveVector = m_cursor - originalPos;
-                    m_cursor = originalPos + GetVectorSnapped45( moveVector );
-                }
-
                 if( updateBBox )
                 {
                     originalBBox = BOX2I();
@@ -492,12 +601,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
 
                     for( EDA_ITEM* item : sel_items )
                     {
-                        BOX2I viewBBOX = item->ViewBBox();
-
-                        if( originalBBox.GetWidth() == 0 && originalBBox.GetHeight() == 0 )
-                            originalBBox = viewBBOX;
-                        else
-                            originalBBox.Merge( viewBBOX );
+                        originalBBox.Merge( item->ViewBBox() );
                     }
 
                     updateBBox = false;
@@ -513,12 +617,14 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                 selection.SetReferencePoint( m_cursor );
 
                 prevPos = m_cursor;
-                totalMovement += movement;
                 bboxMovement += movement;
 
                 // Drag items to the current cursor position
                 for( EDA_ITEM* item : sel_items )
                 {
+                    if( !item->IsBOARD_ITEM() )
+                        continue;
+
                     // Don't double move child items.
                     if( !item->GetParent() || !item->GetParent()->IsSelected() )
                         static_cast<BOARD_ITEM*>( item )->Move( movement );
@@ -587,17 +693,14 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                     // start moving with the reference point attached to the cursor
                     grid.SetAuxAxes( false );
 
-                    if( hv45Mode )
-                    {
-                        VECTOR2I moveVector = m_cursor - originalPos;
-                        m_cursor = originalPos + GetVectorSnapped45( moveVector );
-                    }
-
                     movement = m_cursor - selection.GetReferencePoint();
 
                     // Drag items to the current cursor position
                     for( EDA_ITEM* item : selection )
                     {
+                        if( !item->IsBOARD_ITEM() )
+                            continue;
+
                         // Don't double move footprint pads, fields, etc.
                         if( item->GetParent() && item->GetParent()->IsSelected() )
                             continue;
@@ -627,7 +730,9 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                         }
                     }
 
-                    m_cursor = grid.BestDragOrigin( originalCursorPos, sel_items,
+                    // Use the mouse position over cursor, as otherwise large grids will allow only
+                    // snapping to items that are closest to grid points
+                    m_cursor = grid.BestDragOrigin( originalMousePos, sel_items,
                                                     grid.GetSelectionGrid( selection ),
                                                     &m_selectionTool->GetFilter() );
 
@@ -636,20 +741,43 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                     if( moveWithReference )
                     {
                         selection.SetReferencePoint( pickedReferencePoint );
+
+                        if( hv45Enabled )
+                            grid.SetSnapLineOrigin( selection.GetReferencePoint() );
+
                         controls->ForceCursorPosition( true, pickedReferencePoint );
                         m_cursor = pickedReferencePoint;
                     }
                     else
                     {
-                        // Check if user wants to warp the mouse to origin of moved object
-                        if( !editFrame->GetMoveWarpsCursor() )
-                            m_cursor = originalCursorPos; // No, so use original mouse pos instead
+                        // Don't snap the items on the initial drag start - this would warp
+                        // the object position before the mouse moves. Instead, set up construction
+                        // lines at the current object position and let the user move from there.
 
-                        selection.SetReferencePoint( m_cursor );
-                        grid.SetAuxAxes( true, m_cursor );
+                        // Get the best drag origin (where an item anchor is)
+                        VECTOR2I dragOrigin = m_cursor;
+
+                        // Set the reference point to the drag origin (actual item position)
+                        selection.SetReferencePoint( dragOrigin );
+
+                        // Set up construction/snap lines at the CURRENT position, not a snapped position
+                        if( hv45Enabled )
+                            grid.SetSnapLineOrigin( dragOrigin );
+
+                        grid.SetAuxAxes( true, dragOrigin );
+
+                        // Use the original cursor position if not warping
+                        if( !editFrame->GetMoveWarpsCursor() )
+                            m_cursor = originalCursorPos;
+                        else
+                        {
+                            // Even when warping is enabled, stay at the drag origin initially
+                            // to prevent immediate object movement
+                            m_cursor = dragOrigin;
+                        }
                     }
 
-                    originalPos = m_cursor;
+                    originalPos = selection.GetReferencePoint();
                 }
 
                 // Update variables for bounding box collision calculations
@@ -662,7 +790,8 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                 m_toolMgr->PostEvent( EVENTS::SelectedItemsModified );
             }
 
-            statusPopup.Move( KIPLATFORM::UI::GetMousePosition() + wxPoint( 20, 20 ) );
+            if( statusPopup )
+                statusPopup->Move( KIPLATFORM::UI::GetMousePosition() + wxPoint( 20, 20 ) );
 
             if( enableLocalRatsnest )
                 m_toolMgr->PostAction( PCB_ACTIONS::updateLocalRatsnest, movement );
@@ -677,7 +806,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
         }
         else if( evt->IsClick( BUT_RIGHT ) )
         {
-            m_menu.ShowContextMenu( selection );
+            m_selectionTool->GetToolMenu().ShowContextMenu( selection );
         }
         else if( evt->IsAction( &ACTIONS::undo ) || evt->IsAction( &ACTIONS::doDelete ) )
         {
@@ -686,6 +815,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
         }
         else if( evt->IsAction( &ACTIONS::duplicate ) || evt->IsAction( &ACTIONS::cut ) )
         {
+            wxBell();
         }
         else if( evt->IsAction( &PCB_ACTIONS::rotateCw )
                 || evt->IsAction( &PCB_ACTIONS::rotateCcw )
@@ -723,6 +853,8 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                     originalPos = nextItem->GetPosition();
                     m_selectionTool->AddItemToSel( nextItem );
                     selection.SetReferencePoint( originalPos );
+                    if( hv45Enabled )
+                        grid.SetSnapLineOrigin( selection.GetReferencePoint() );
 
                     sel_items.clear();
                     sel_items.push_back( nextItem );
@@ -748,15 +880,19 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
         }
         else if( evt->IsAction( &PCB_ACTIONS::toggleHV45Mode ) )
         {
-            hv45Mode = !hv45Mode;
-            displayConstraintsMessage( hv45Mode );
-            evt->SetPassEvent( false );
+            evt->SetPassEvent( true );
+        }
+        else if( evt->IsAction( &ACTIONS::increment ) )
+        {
+            m_toolMgr->RunSynchronousAction( ACTIONS::increment, aCommit,
+                                             evt->Parameter<ACTIONS::INCREMENT>() );
         }
         else if( ZONE_FILLER_TOOL::IsZoneFillAction( evt )
                  || evt->IsAction( &PCB_ACTIONS::moveExact )
                  || evt->IsAction( &PCB_ACTIONS::moveWithReference )
                  || evt->IsAction( &PCB_ACTIONS::copyWithReference )
                  || evt->IsAction( &PCB_ACTIONS::positionRelative )
+                 || evt->IsAction( &PCB_ACTIONS::positionRelativeInteractively )
                  || evt->IsAction( &ACTIONS::redo ) )
         {
             wxBell();
@@ -808,7 +944,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
     // Remove the dynamic ratsnest from the screen
     m_toolMgr->RunAction( PCB_ACTIONS::hideLocalRatsnest );
 
-    editFrame->PopTool( aEvent );
+    editFrame->PopTool( pushedEvent );
     editFrame->GetCanvas()->SetCurrentCursor( KICURSOR::ARROW );
 
     return !restore_state;
