@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2020-2022 KiCad Developers.
+ * Copyright The KiCad Developers.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -65,6 +65,22 @@ PNS_LOG_VIEWER_OVERLAY::PNS_LOG_VIEWER_OVERLAY( KIGFX::GAL* aGal )
 }
 
 
+void PNS_LOG_VIEWER_OVERLAY::AnnotatedPolyset( const SHAPE_POLY_SET& aPolyset, std::string aName,
+                                               bool aShowVertexNumbers )
+{
+    for( int i = 0; i < aPolyset.OutlineCount(); i++ )
+    {
+        if( i == 0 && !aName.empty() )
+            AnnotatedPolyline( aPolyset.COutline( i ), aName );
+        else
+            AnnotatedPolyline( aPolyset.COutline( i ), "" );
+
+        for( int j = 0; j < aPolyset.HoleCount( i ); j++ )
+            AnnotatedPolyline( aPolyset.CHole( i, j ), "" );
+    }
+}
+
+
 void PNS_LOG_VIEWER_OVERLAY::AnnotatedPolyline( const SHAPE_LINE_CHAIN& aL, std::string name,
                                                 bool aShowVertexNumbers )
 {
@@ -117,12 +133,14 @@ void PNS_LOG_VIEWER_OVERLAY::DrawAnnotations()
 
 
 PNS_LOG_VIEWER_FRAME::PNS_LOG_VIEWER_FRAME( wxFrame* frame ) :
-        PNS_LOG_VIEWER_FRAME_BASE( frame ), m_rewindIter( 0 ), m_reporter( &m_consoleLog )
+        PNS_LOG_VIEWER_FRAME_BASE( frame ), m_rewindIter( 0 )
 {
     LoadSettings();
-    createView( this, PCB_DRAW_PANEL_GAL::GAL_TYPE_OPENGL );
+    createView( m_mainSplitter, PCB_DRAW_PANEL_GAL::GAL_TYPE_OPENGL );
 
-    m_viewSizer->Add( m_galPanel.get(), 1, wxEXPAND, 5 );
+    m_reporter.reset( new WX_TEXT_CTRL_REPORTER( m_consoleText ) );
+    m_galPanel->SetParent( m_mainSplitter );
+    m_mainSplitter->SplitHorizontally( m_galPanel.get(), m_panelProps );
 
     Layout();
 
@@ -329,11 +347,20 @@ void PNS_LOG_VIEWER_FRAME::LoadLogFile( const wxString& aFile )
 {
     std::unique_ptr<PNS_LOG_FILE> logFile( new PNS_LOG_FILE );
 
-    if( logFile->Load( wxFileName( aFile ), &m_reporter ) )
+    wxFileName logFn( aFile );
+    logFn.MakeAbsolute();
+
+    if( logFile->Load( logFn, m_reporter.get() ) )
         SetLogFile( logFile.release() );
 }
 
 
+void PNS_LOG_VIEWER_FRAME::updateViewerIface()
+{
+    m_viewerIface = std::make_shared<PNS_VIEWER_IFACE>( m_board );
+}
+
+
 void PNS_LOG_VIEWER_FRAME::SetLogFile( PNS_LOG_FILE* aLog )
 {
     m_logPlayer.reset( new PNS_LOG_PLAYER );
@@ -341,7 +368,7 @@ void PNS_LOG_VIEWER_FRAME::SetLogFile( PNS_LOG_FILE* aLog )
     m_logFile.reset( aLog );
 
     SetBoard( m_logFile->GetBoard() );
-
+    updateViewerIface();
     m_logPlayer->ReplayLog( m_logFile.get(), 0, 0, -1, true );
 
     auto dbgd = m_logPlayer->GetDebugDecorator();
@@ -364,13 +391,14 @@ void PNS_LOG_VIEWER_FRAME::SetLogFile( PNS_LOG_FILE* aLog )
     drawLoggedItems( m_rewindIter );
     updateDumpPanel( m_rewindIter );
     updatePnsPreviewItems( m_rewindIter );
+
 }
 
 
 void PNS_LOG_VIEWER_FRAME::SetBoard2( std::shared_ptr<BOARD> aBoard )
 {
     SetBoard( aBoard );
-
+    updateViewerIface();
     auto extents = m_board->GetBoundingBox();
 
     BOX2D bbd;
@@ -402,7 +430,7 @@ void PNS_LOG_VIEWER_FRAME::onSaveAs( wxCommandEvent& event )
 {
     if( !m_logFile )
     {
-        DisplayError( this, wxT( "No log file Loaded!" ) );
+        DisplayErrorMessage( this, wxT( "No log file Loaded!" ) );
         return;
     }
 
@@ -417,7 +445,7 @@ void PNS_LOG_VIEWER_FRAME::onSaveAs( wxCommandEvent& event )
 
         wxASSERT_MSG( create_me.IsAbsolute(), wxS( "wxFileDialog returned non-absolute path" ) );
 
-        m_logFile->SaveLog( create_me, &m_reporter );
+        m_logFile->SaveLog( create_me, m_reporter.get() );
         m_mruPath = create_me.GetPath();
     }
 
@@ -905,10 +933,10 @@ void PNS_LOG_VIEWER_FRAME::updatePnsPreviewItems( int iter )
 
     for( auto& ent : entries )
     {
-        if ( ent.isHideOp )
+        if ( ent.m_isHideOp )
         {
 
-            auto parent = ent.item->Parent();
+            auto parent = ent.m_item->Parent();
             if( parent )
             {
 
@@ -917,8 +945,8 @@ void PNS_LOG_VIEWER_FRAME::updatePnsPreviewItems( int iter )
         }
         else
         {
-            ROUTER_PREVIEW_ITEM* pitem = new ROUTER_PREVIEW_ITEM( ent.item, view );
-            pitem->Update( ent.item );
+            ROUTER_PREVIEW_ITEM* pitem = new ROUTER_PREVIEW_ITEM( ent.m_item, m_viewerIface.get(), view );
+            pitem->Update( ent.m_item );
             m_previewItems->Add(pitem);
     //        printf("DBG vadd %p total %d\n", pitem, m_previewItems->GetSize() );
         }
@@ -933,6 +961,12 @@ void PNS_LOG_VIEWER_FRAME::updatePnsPreviewItems( int iter )
     //view->UpdateAllItems( KIGFX::ALL );
 }
 
+REPORTER* PNS_LOG_VIEWER_FRAME::GetConsoleReporter()
+{
+    return m_reporter.get();
+}
+
+
 #if 0
 
 static BOARD* loadBoard( const std::string& filename )
